'''
ypya.payload
'''
import json
import logging
import traceback


class GELFData(object):
    '''
    data type adapter for GELF
    '''
    SYSLOG_LEVELS = {
        logging.CRITICAL: 2,
        logging.ERROR: 3,
        logging.WARNING: 4,
        logging.INFO: 6,
        logging.DEBUG: 7,
    }

    SPECIAL_NAME_PREFIX = '_'
    GELF_VERSION_NUMBER = '1.1'

    BASIC_FIELDS = (
        'version',
        'timestamp',
        'host',
        'facility',
        'level',
        'short_message',
        'full_message',
    )
    OPTIONAL_FIELDS = (
        'level_name',
        '_logger',
        'file',
        'line',
        '_function',
        '_pid',
        '_thread_name',
        '_process_name',
    )

    def __init__(self, record, parent=None):
        '''
        :param record: (logging.LogRecord)
        :param parent: (obj)
        '''
        assert record is not None, 'invalid argument value'
        self.instance = record
        self.parent = parent

    @property
    def data(self):
        '''
        :return: (string/unicode)
        '''
        self.cache = dict()
        self.formatter = getattr(
            self.parent,
            'formatter',
            None
        )

        self.collect_basic_fields()
        self.add_optional_fields()

        # rval;
        rval = json.dumps(self.cache)
        return rval

    def collect_basic_fields(self):
        '''
        :return: (dict)
        '''
        self.cache['version'] = self.GELF_VERSION_NUMBER
        self.cache['timestamp'] = self.instance.created
        self.cache['level'] = self.SYSLOG_LEVELS.get(
            self.instance.levelno,
            self.instance.levelno
        )
        self.cache['host'] = self.parent.sender_host
        originator = self.parent.sender_name or self.instance.name
        self.cache['facility'] = originator

        self.cache['short_message'] = self.extract_short_message(
            self.instance
        )
        self.cache['full_message'] = self.extract_full_message(
            self.instance
        )

        return self.cache

    def add_optional_fields(self):
        '''
        :return: (dict)
        '''
        if self.parent.use_level_name:
            self.cache['level_name'] = logging.getLevelName(
                self.instance.levelno
            )

        if self.parent.sender_name is not None:
            self.cache['_logger'] = self.instance.name

        if self.parent.add_debug_field:
            self.cache['file'] = self.instance.pathname
            self.cache['line'] = self.instance.lineno
            self.cache['_function'] = self.instance.funcName
            self.cache['_pid'] = self.instance.process
            self.cache['_thread_name'] = self.instance.threadName

            pn = self.instance.processName
            if pn is not None:
                self.cache['_process_name'] = pn
        return self.cache

    def extract_short_message(self, obj):
        '''
        :param obj: (logging.LogRecord)
        :return: (string)
        '''
        if self.formatter is not None:
            msg = formatter.format(obj)
        else:
            msg = obj.getMessage()
        return msg

    def extract_full_message(self, obj):
        '''
        :param obj: (logging.LogRecord)
        :return: (string)
        '''
        if self.formatter is not None:
            msg = self.formatter.format(obj)
        else:
            if obj.exc_info:
                items = traceback.format_exception(*obj.exc_info)
                msg = '\n'.join(items)
            elif obj.exc_text:
                msg = obj.exc_text
            else:
                msg = obj.getMessage()
        return msg



'''
ypya.base
'''


class YBase(object):
    '''
    basic declaration
    '''

    INCLUDE_DEBUG_FIELD = True
    INCLUDE_EXTRA_FIELD = True
    INCLUDE_FULLY_QUALIFIED_NAME = False
    INCLUDE_LEVEL_SYMBOLIC_NAME = False

    def makePickle(self, record):
        '''
        public method

        :param record: (logging.LogRecord)
        :return: (stringified JSON)
        '''
        obj = self.payload_cls(
            record,
            parent=self
        )
        return obj.data



'''
ypya.utils
'''
from logging import Filter
from logging.handlers import SocketHandler
import socket


from .base import YBase
from .payload import GELFData
from .transport import AMQPTransport


class YExclusion(Filter):
    '''
    A simple exclusion filter based on the name of log message sending facility
    '''
    NS_SEP = '.'

    def __init__(self, name):
        assert (name is not None) and (len(name) > 0 ), 'invalid value'
        self.name = name

    def _get_ns_path_root(self, val):
        '''
        :param val: (string)
        :return: (string)
        '''
        tokens = val.split(self.NS_SEP, 1)
        return tokens[0]

    def filter(self, record):
        '''
        public method

        :param record: (obj)
        :return: (boolean)
        '''
        match = (self._get_ns_path_root(record.name) == self.name)
        return not(match)


class YLogHandler(YBase, SocketHandler):
    '''
    A simple logging handler
    '''

    def __init__(
            self,
            broker_host=AMQPTransport.DEFAULT_HOSTNAME,
            broker_port=AMQPTransport.DEFAULT_PORT,
            broker_auth=AMQPTransport.DEFAULT_CREDENTIAL,
            vhost=AMQPTransport.DEFAULT_VHOST,
            exchange_name=AMQPTransport.DEFAULT_EXCHANGE_NAME,
            exchange_type=AMQPTransport.DEFAULT_EXCHANGE_TYPE,
            routing_key=AMQPTransport.DEFAULT_ROUTING_KEY,
            facility=None,
            localname=None,
            fqdn=YBase.INCLUDE_FULLY_QUALIFIED_NAME,
            add_level_name=YBase.INCLUDE_LEVEL_SYMBOLIC_NAME,
            debugging_fields=YBase.INCLUDE_DEBUG_FIELD,
            extra_fields=YBase.INCLUDE_EXTRA_FIELD,
            deep_sanitize=False
    ):

        self.credentials = broker_auth
        self.hostname = broker_host
        self.port = broker_port

        self.vhost = vhost
        self.exchange_name = exchange_name
        self.exchange_type = exchange_type
        self.routing_key = routing_key

        self.transport = AMQPTransport()

        self.sender_host = localname
        self.sender_name = facility
        self.use_fully_qualified_name = fqdn
        self.use_level_name = add_level_name
        self.add_debug_field = debugging_fields
        self.add_extra_field = extra_fields
        self.safe_but_slow = deep_sanitize

        self.payload_cls = GELFData
        SocketHandler.__init__(
            self,
            host=self.hostname,
            port=self.port
        )
        self.addFilter(
            YExclusion(self.transport.backend_name)
        )

    def setFormatter(self, fmt):
        '''
        public method

        :param fmt: (`logging.Formatter`)
        '''
        self.formatter = fmt

    def makeSocket(self):
        '''
        public method
        '''
        conn_args = {
            'host': '{h}:{p}'.format(
                h=self.hostname,
                p=self.port
            ),
            'userid': self.credentials[0],
            'password': self.credentials[1],
            'vhost': self.vhost,
            'insist': False,
        }

        local_name_unset = (
            (self.sender_host is None)
            or
            (len(self.sender_host) == 0)
        )
        if local_name_unset:
            if self.use_fully_qualified_name:
                refresh_host = socket.getfqdn()
            else:
                refresh_host = socket.gethostname()
            self.sender_host = refresh_host

        return self.transport.getSocket(self, conn_args)
